package net.wanji.web.kafka;

import com.alibaba.fastjson.JSONObject;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.kafka.clients.consumer.Consumer;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author duanruiming
 * @date 2023/03/09 16:03
 */
@Slf4j
@Component
public class ConsumerHandler implements KafkaListenerErrorHandler {

    /**
     * kafka数据缓存
     * key:topic value:data
     */
    private static final Map<String, JNMatchResultMiniData> carInfoMap = new ConcurrentHashMap<>();

    /**
     * 通过消息主题获取消息信息
     *
     * @param topic
     * @return
     */
    public JNMatchResultMiniData getTopicMessage(String topic) {
        if (carInfoMap.isEmpty()) {
            return null;
        }
        return carInfoMap.get(topic);
    }

    @KafkaListener(topics = {"CSMatchResultMiniData_0"})
    public void receiveCarInfoData(ConsumerRecord<Object, Object> record, Acknowledgment acknowledgment) {
        try {
            log.info("收到kafka消息，消息主题：" + record.topic());
            log.info("收到kafka消息，消息内容：" + record.value());
            JNMatchResultMiniData jnMatchResultMiniData = JSONObject.parseObject((String)record.value(), JNMatchResultMiniData.class);
            carInfoMap.put(record.topic(), jnMatchResultMiniData);
            acknowledgment.acknowledge();
        } catch (Exception e) {
            log.error("监听主题消息异常， 异常信息", e);
        }
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }

}
